@@ -29,6 +29,7 @@
 
 #include "facebook.h"
 #include "facebook-util.h"
+#include "facebook-contact-view.h"
 #include "facebook-item-view.h"
 
 #include <json-glib/json-glib.h>
@@ -46,10 +47,12 @@
 #include <interfaces/sw-status-update-ginterface.h>
 #include <interfaces/sw-photo-upload-ginterface.h>
 #include <interfaces/sw-video-upload-ginterface.h>
+#include <interfaces/sw-contacts-query-ginterface.h>
 #include <interfaces/sw-query-ginterface.h>
 #include <interfaces/sw-collections-ginterface.h>
 
 static void initable_iface_init (gpointer g_iface, gpointer iface_data);
+static void contacts_query_iface_init (gpointer g_iface, gpointer iface_data);
 static void query_iface_init (gpointer g_iface, gpointer iface_data);
 static void avatar_iface_init (gpointer g_iface, gpointer iface_data);
 static void status_update_iface_init (gpointer g_iface, gpointer iface_data);
@@ -62,6 +65,8 @@ G_DEFINE_TYPE_WITH_CODE (SwServiceFacebook,
                          SW_TYPE_SERVICE,
                          G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
                                                 initable_iface_init)
+                         G_IMPLEMENT_INTERFACE (SW_TYPE_CONTACTS_QUERY_IFACE,
+                                                contacts_query_iface_init)
                          G_IMPLEMENT_INTERFACE (SW_TYPE_QUERY_IFACE,
                                                 query_iface_init)
                          G_IMPLEMENT_INTERFACE (SW_TYPE_AVATAR_IFACE,
@@ -121,6 +126,7 @@ get_static_caps (SwService *service)
     HAS_UPDATE_STATUS_IFACE,
     HAS_AVATAR_IFACE,
     HAS_BANISHABLE_IFACE,
+    HAS_CONTACTS_QUERY_IFACE,
     HAS_QUERY_IFACE,
     HAS_PHOTO_UPLOAD_IFACE,
     HAS_VIDEO_UPLOAD_IFACE,
@@ -461,6 +467,42 @@ initable_iface_init (gpointer g_iface, gpointer iface_data)
 }
 
 /* Query interface */
+static void
+_facebook_contacts_query_open_view (SwContactsQueryIface  *self,
+                                    const gchar           *query,
+                                    GHashTable            *params,
+                                    DBusGMethodInvocation *context)
+{
+  SwServiceFacebookPrivate *priv = GET_PRIVATE (self);
+  SwContactView *contact_view;
+  const gchar *object_path;
+
+  g_debug ("query = '%s'", query);
+  if (!g_str_equal (query, "people"))
+  {
+    dbus_g_method_return_error (context,
+                                g_error_new (SW_SERVICE_ERROR,
+                                             SW_SERVICE_ERROR_INVALID_QUERY,
+                                             "Query '%s' is invalid",
+                                             query));
+    return;
+  }
+
+  contact_view = g_object_new (SW_TYPE_FACEBOOK_CONTACT_VIEW,
+                               "service", self,
+                               "proxy", priv->proxy,
+                               "query", query,
+                               "params", params,
+                               NULL);
+  object_path = sw_contact_view_get_object_path (contact_view);
+
+  /* Ensure the object gets disposed when the client goes away */
+  sw_client_monitor_add (dbus_g_method_get_sender (context),
+                         (GObject *) contact_view);
+
+  sw_contacts_query_iface_return_from_open_view (context, object_path);
+}
+
 static void
 _facebook_query_open_view (SwQueryIface          *self,
                            const gchar           *query,
@@ -488,6 +530,14 @@ _facebook_query_open_view (SwQueryIface          *self,
   sw_query_iface_return_from_open_view (context, object_path);
 }
 
+static void
+contacts_query_iface_init (gpointer g_iface, gpointer iface_data)
+{
+  SwContactsQueryIfaceClass *klass = (SwContactsQueryIfaceClass*) g_iface;
+
+  sw_contacts_query_iface_implement_open_view (klass, _facebook_contacts_query_open_view);
+}
+
 static void
 query_iface_init (gpointer g_iface, gpointer iface_data)
 {
@@ -590,12 +640,12 @@ status_update_iface_init (gpointer g_iface,
 }
 
 static gint
-_upload_file (SwServiceFacebook           *self,
-              MediaType                    upload_type,
-              const gchar                 *filename,
-              GHashTable                  *fields,
-              RestProxyCallAsyncCallback   upload_cb,
-              GError                     **error)
+_upload_file (SwServiceFacebook            *self,
+              MediaType                     upload_type,
+              const gchar                  *filename,
+              GHashTable                   *fields,
+              RestProxyCallUploadCallback   upload_cb,
+              GError                      **error)
 {
   SwServiceFacebookPrivate *priv = self->priv;
   RestProxyCall *call = NULL;
@@ -681,11 +731,11 @@ _upload_file (SwServiceFacebook           *self,
 
   opid = sw_next_opid ();
 
-  rest_proxy_call_async (call,
-                         upload_cb,
-                         G_OBJECT (self),
-                         GINT_TO_POINTER (opid),
-                         NULL);
+  rest_proxy_call_upload (call,
+                          upload_cb,
+                          G_OBJECT (self),
+                          GINT_TO_POINTER (opid),
+                          NULL);
 
 
  OUT:
@@ -703,6 +753,8 @@ _upload_file (SwServiceFacebook           *self,
 
 static void
 _upload_photo_cb (RestProxyCall *call,
+                  gsize          total,
+                  gsize          uploaded,
                   const GError  *error,
                   GObject       *weak_object,
                   gpointer       user_data)
@@ -714,7 +766,9 @@ _upload_photo_cb (RestProxyCall *call,
     sw_photo_upload_iface_emit_photo_upload_progress (facebook, opid, -1,
                                                       error->message);
   } else {
-    sw_photo_upload_iface_emit_photo_upload_progress (facebook, opid, 100, "");
+    gint percent = (gdouble) uploaded / (gdouble) total * 100;
+    sw_photo_upload_iface_emit_photo_upload_progress (facebook, opid, percent,
+                                                      "");
   }
 }
 
@@ -729,7 +783,7 @@ _facebook_photo_upload_upload_photo (SwPhotoUploadIface    *self,
   GError *error = NULL;
 
   opid = _upload_file (facebook, PHOTO, filename, fields,
-                       (RestProxyCallAsyncCallback) _upload_photo_cb, &error);
+                       (RestProxyCallUploadCallback) _upload_photo_cb, &error);
 
   if (error) {
     dbus_g_method_return_error (context, error);
@@ -752,6 +806,8 @@ photo_upload_iface_init (gpointer g_iface,
 
 static void
 _upload_video_cb (RestProxyCall *call,
+                  gsize          total,
+                  gsize          uploaded,
                   const GError  *error,
                   GObject       *weak_object,
                   gpointer       user_data)
@@ -763,7 +819,9 @@ _upload_video_cb (RestProxyCall *call,
     sw_video_upload_iface_emit_video_upload_progress (facebook, opid, -1,
                                                       error->message);
   } else {
-    sw_video_upload_iface_emit_video_upload_progress (facebook, opid, 100, "");
+    gint percent = (gdouble) uploaded / (gdouble) total * 100;
+    sw_video_upload_iface_emit_video_upload_progress (facebook, opid, percent,
+                                                      "");
   }
 }
 
@@ -778,7 +836,7 @@ _facebook_video_upload_upload_video (SwVideoUploadIface    *self,
   GError *error = NULL;
 
   opid = _upload_file (facebook, VIDEO, filename, fields,
-                       (RestProxyCallAsyncCallback) _upload_video_cb, &error);
+                       (RestProxyCallUploadCallback) _upload_video_cb, &error);
 
   if (error) {
     dbus_g_method_return_error (context, error);
